@@ -1,6 +1,6 @@
 # -*- perl -*- Lintian::Index::Ar
 #
-# Copyright © 2020 Felix Lechner
+# Copyright (C) 2020 Felix Lechner
 #
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
@@ -31,6 +31,8 @@ use Lintian::IPC::Run3 qw(safe_qx);
 use Moo::Role;
 use namespace::clean;
 
+use File::LibMagic;
+
 const my $EMPTY => q{};
 const my $NEWLINE => qq{\n};
 
@@ -59,13 +61,19 @@ sub add_ar {
 
     my $savedir = getcwd;
     chdir($self->basedir)
-      or die encode_utf8('Cannot change to directory ' . $self->basedir);
+      or die encode_utf8(
+        $self->identifier . ': Cannot change to directory ' . $self->basedir);
 
     my $errors = $EMPTY;
 
-    my @archives
-      = grep { $_->name =~ / [.]a $/msx && $_->is_regular_file }
-      @{$self->sorted_list};
+    my $magic = File::LibMagic->new;
+
+    my @archives= grep {
+        $_->name =~ / [.]a $/msx
+          && $_->is_regular_file
+          && $magic->info_from_filename($_->name)->{mime_type} eq
+          'application/x-archive'
+    }@{$self->sorted_list};
 
     for my $archive (@archives) {
 
@@ -99,7 +107,8 @@ sub add_ar {
     }
 
     chdir($savedir)
-      or die encode_utf8("Cannot change to directory $savedir");
+      or die encode_utf8(
+        $self->identifier . ": Cannot change to directory $savedir");
 
     return $errors;
 }
